@@ -26,16 +26,15 @@
 #include <haproxy/buf-t.h>
 #include <haproxy/dynbuf-t.h>
 #include <haproxy/freq_ctr-t.h>
-#include <haproxy/hlua-t.h>
 #include <haproxy/obj_type-t.h>
-#include <haproxy/proxy-t.h>
-#include <haproxy/stream-t.h>
 #include <haproxy/xref-t.h>
 
 /* flags for appctx->state */
 #define APPLET_WANT_DIE     0x01  /* applet was running and requested to die */
 
 struct appctx;
+struct stream;
+struct proxy;
 
 /* Applet descriptor */
 struct applet {
@@ -67,7 +66,6 @@ struct appctx {
 	int cli_severity_output;        /* used within the cli_io_handler to format severity output of informational feedback */
 	int cli_level;              /* the level of CLI which can be lowered dynamically */
 	struct buffer_wait buffer_wait; /* position in the list of objects waiting for a buffer */
-	unsigned long thread_mask;      /* mask of thread IDs authorized to process the applet */
 	struct task *t;                  /* task associated to the applet */
 	struct freq_ctr call_rate;       /* appctx call rate */
 	struct list wait_entry;          /* entry in a list of waiters for an event (e.g. ring events) */
@@ -175,14 +173,25 @@ struct appctx {
 			int flags;           /* non-zero if "dict" dump requested */
 		} cfgpeers;
 		struct {
-			char *err;
+			char *path;
 			struct ckch_store *old_ckchs;
 			struct ckch_store *new_ckchs;
 			struct ckch_inst *next_ckchi;
+
+			struct ckch_inst_link *next_ckchi_link;
+			struct cafile_entry *old_cafile_entry;
+			struct cafile_entry *new_cafile_entry;
+
+			struct cafile_entry *old_crlfile_entry;
+			struct cafile_entry *new_crlfile_entry;
+			int cafile_type; /* either CA or CRL, depending on the current command */
 		} ssl;
 		struct {
 			void *ptr;
 		} sft; /* sink forward target */
+		struct {
+			struct httpclient *ptr;
+		} httpclient;
 
 		/* NOTE: please add regular applet contexts (ie: not
 		 * CLI-specific ones) above, before "cli".
