@@ -37,30 +37,6 @@
 #endif
 #endif
 
-/* This is used to test if a macro is defined and equals 1. The principle is
- * that the macro is passed as a value and its value concatenated to the word
- * "comma_for_one" to form a new macro name. The macro "comma_for_one1" equals
- * one comma, which, once used in an argument, will shift all of them by one,
- * so that we can use this to concatenate both a 1 and a 0 and always pick the
- * second one.
- */
-#define comma_for_one1 ,
-#define _____equals_1(x, y, ...) (y)
-#define ____equals_1(x, ...) _____equals_1(x, 0)
-#define ___equals_1(x)       ____equals_1(comma_for_one ## x 1)
-#define __equals_1(x)        ___equals_1(x)
-
-/* gcc 5 and clang 3 brought __has_attribute(), which is not well documented in
- * the case of gcc, but is convenient since handled at the preprocessor level.
- * In both cases it's possible to test for __has_attribute() using ifdef. When
- * not defined we remap this to the __has_attribute_<name> macro so that we'll
- * later be able to implement on a per-compiler basis those which are missing,
- * by defining __has_attribute_<name> to 1.
- */
-#ifndef __has_attribute
-#define __has_attribute(x) __equals_1(__has_attribute_ ## x)
-#endif
-
 #if !defined(__GNUC__)
 /* Some versions of glibc irresponsibly redefine __attribute__() to empty for
  * non-gcc compilers, and as such, silently break all constructors with other
@@ -82,12 +58,10 @@
 #endif
 #endif
 
-#ifndef __maybe_unused
 /* silence the "unused" warnings without having to place painful #ifdefs.
  * For use with variables or functions.
  */
 #define __maybe_unused __attribute__((unused))
-#endif
 
 /* These macros are used to declare a section name for a variable.
  * WARNING: keep section names short, as MacOS limits them to 16 characters.
@@ -114,7 +88,7 @@
 
 #endif // USE_OBSOLETE_LINKER
 
-/* Declare a symbol as weak if possible, otherwise global. Since we don't want to
+/* Declare a symbol as global and if possible, as weak. Since we don't want to
  * error on multiple definitions, the symbol is declared weak. On MacOS ".weak"
  * does not exist and we must continue to use ".globl" instead. Note that
  * ".global" is to be avoided on other platforms as llvm complains about it
@@ -123,15 +97,11 @@
  * anyway (and most likely it will only work with !USE_OBSOLETE_LINKER).
  */
 #if defined(__APPLE__)
-#  define __HA_WEAK(sym)   __asm__(".globl " #sym)
+#  define __HA_GLOBL(sym)   __asm__(".globl " #sym)
 #else
-#  define __HA_WEAK(sym)   __asm__(".weak " #sym)
+#  define __HA_GLOBL(sym)   __asm__(".weak " #sym)
 #endif
-#define HA_WEAK(sym)    __HA_WEAK(sym)
-
-/* declare a symbol as global */
-#define __HA_GLOBL(sym)   __asm__(".globl " #sym)
-#define HA_GLOBL(sym)     __HA_GLOBL(sym)
+#define HA_GLOBL(sym)    __HA_GLOBL(sym)
 
 /* use this attribute on a variable to move it to the read_mostly section */
 #if !defined(__read_mostly)
@@ -150,7 +120,7 @@
 #if defined(__GNUC__) && (__GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 5))
 #define my_unreachable() __builtin_unreachable()
 #else
-#define my_unreachable() do { } while (1)
+#define my_unreachable()
 #endif
 #endif
 
@@ -216,6 +186,23 @@
 #endif
 #endif
 
+/* Linux-like "container_of". It returns a pointer to the structure of type
+ * <type> which has its member <name> stored at address <ptr>.
+ */
+#ifndef container_of
+#define container_of(ptr, type, name) ((type *)(((void *)(ptr)) - ((long)&((type *)0)->name)))
+#endif
+
+/* returns a pointer to the structure of type <type> which has its member <name>
+ * stored at address <ptr>, unless <ptr> is 0, in which case 0 is returned.
+ */
+#ifndef container_of_safe
+#define container_of_safe(ptr, type, name) \
+	({ void *__p = (ptr); \
+		__p ? (type *)(__p - ((long)&((type *)0)->name)) : (type *)0; \
+	})
+#endif
+
 /* Some architectures have a double-word CAS, sometimes even dual-8 bytes.
  * Some architectures support unaligned accesses, others are fine with them
  * but only for non-atomic operations. Also mention those supporting unaligned
@@ -375,15 +362,4 @@
 #define __has_feature(x) 0
 #endif
 
-/* gcc 15 throws warning if fixed-size char array does not contain a terminating
- * NUL. gcc has an attribute 'nonstring', which allows to suppress this warning
- * for such array declarations. But it's not the case for clang and other
- * compilers.
- */
-#if __has_attribute(nonstring)
-#define __nonstring __attribute__ ((nonstring))
-#else
-#define __nonstring
-#endif
-
 #endif /* _HAPROXY_COMPILER_H */
