@@ -1,10 +1,12 @@
 
 local vtc_port = 0
 local vtc_port2 = 0
+local vtc_port3 = 0
 
 core.register_service("fakeserv", "http", function(applet)
 	vtc_port = applet.headers["vtcport"][0]
 	vtc_port2 = applet.headers["vtcport2"][0]
+	vtc_port3 = applet.headers["vtcport3"][0]
 	core.Info("APPLET START")
 	local response = "OK"
 	applet:add_header("Server", "haproxy/webstats")
@@ -38,6 +40,10 @@ local function cron()
 	local httpclient2 = core.httpclient()
 	local response2 = httpclient2:post{url="http://127.0.0.1:" .. vtc_port2, body=body}
 
+	core.Info("Third httpclient request")
+	local httpclient3 = core.httpclient()
+	local response3 = httpclient3:get{url="http://127.0.0.1:" .. vtc_port3, headers={ [ "Host" ] = { "foobar.haproxy.local" } }}
+
 end
 
 core.register_task(cron)
