@@ -37,7 +37,7 @@
 #include <haproxy/stream.h>
 #include <haproxy/stream_interface.h>
 #include <haproxy/task.h>
-#include <haproxy/time.h>
+#include <haproxy/ticks.h>
 #include <haproxy/tools.h>
 
 
@@ -47,6 +47,7 @@
  */
 int frontend_accept(struct stream *s)
 {
+	const struct sockaddr_storage *src, *dst;
 	struct session *sess = s->sess;
 	struct connection *conn = objt_conn(sess->origin);
 	struct listener *l = sess->listener;
@@ -60,35 +61,38 @@ int frontend_accept(struct stream *s)
 				if (!(s->logs.logwait &= ~(LW_CLIP|LW_INIT)))
 					s->do_log(s);
 		}
-		else if (conn && !conn_get_src(conn)) {
-			send_log(fe, LOG_INFO, "Connect from unknown source to listener %d (%s/%s)\n",
-				 l->luid, fe->id, (fe->mode == PR_MODE_HTTP) ? "HTTP" : "TCP");
-		}
 		else if (conn) {
-			char pn[INET6_ADDRSTRLEN], sn[INET6_ADDRSTRLEN];
-			int port;
+			src = si_src(&s->si[0]);
+			if (!src)
+				send_log(fe, LOG_INFO, "Connect from unknown source to listener %d (%s/%s)\n",
+					 l->luid, fe->id, (fe->mode == PR_MODE_HTTP) ? "HTTP" : "TCP");
+			else {
+				char pn[INET6_ADDRSTRLEN], sn[INET6_ADDRSTRLEN];
+				int port;
 
-			switch (addr_to_str(conn->src, pn, sizeof(pn))) {
-			case AF_INET:
-			case AF_INET6:
-				if (conn_get_dst(conn)) {
-					addr_to_str(conn->dst, sn, sizeof(sn));
-					port = get_host_port(conn->dst);
-				} else {
-					strcpy(sn, "undetermined address");
-					port = 0;
+				switch (addr_to_str(src, pn, sizeof(pn))) {
+				case AF_INET:
+				case AF_INET6:
+					dst = si_dst(&s->si[0]);
+					if (dst) {
+						addr_to_str(dst, sn, sizeof(sn));
+						port = get_host_port(dst);
+					} else {
+						strcpy(sn, "undetermined address");
+						port = 0;
+					}
+					send_log(fe, LOG_INFO, "Connect from %s:%d to %s:%d (%s/%s)\n",
+						 pn, get_host_port(src),
+						 sn, port,
+						 fe->id, (fe->mode == PR_MODE_HTTP) ? "HTTP" : "TCP");
+					break;
+				case AF_UNIX:
+					/* UNIX socket, only the destination is known */
+					send_log(fe, LOG_INFO, "Connect to unix:%d (%s/%s)\n",
+						 l->luid,
+						 fe->id, (fe->mode == PR_MODE_HTTP) ? "HTTP" : "TCP");
+					break;
 				}
-				send_log(fe, LOG_INFO, "Connect from %s:%d to %s:%d (%s/%s)\n",
-					 pn, get_host_port(conn->src),
-					 sn, port,
-					 fe->id, (fe->mode == PR_MODE_HTTP) ? "HTTP" : "TCP");
-				break;
-			case AF_UNIX:
-				/* UNIX socket, only the destination is known */
-				send_log(fe, LOG_INFO, "Connect to unix:%d (%s/%s)\n",
-					 l->luid,
-					 fe->id, (fe->mode == PR_MODE_HTTP) ? "HTTP" : "TCP");
-				break;
 			}
 		}
 	}
@@ -109,22 +113,23 @@ int frontend_accept(struct stream *s)
 			}
 		}
 
-		if (!conn_get_src(conn)) {
+		src = si_src(&s->si[0]);
+		if (!src) {
 			chunk_printf(&trash, "%08x:%s.accept(%04x)=%04x from [listener:%d] ALPN=%s\n",
-			             s->uniq_id, fe->id, (unsigned short)l->fd, (unsigned short)conn->handle.fd,
+			             s->uniq_id, fe->id, (unsigned short)l->rx.fd, (unsigned short)conn->handle.fd,
 			             l->luid, alpn);
 		}
-		else switch (addr_to_str(conn->src, pn, sizeof(pn))) {
+		else switch (addr_to_str(src, pn, sizeof(pn))) {
 		case AF_INET:
 		case AF_INET6:
 			chunk_printf(&trash, "%08x:%s.accept(%04x)=%04x from [%s:%d] ALPN=%s\n",
-			             s->uniq_id, fe->id, (unsigned short)l->fd, (unsigned short)conn->handle.fd,
-			             pn, get_host_port(conn->src), alpn);
+			             s->uniq_id, fe->id, (unsigned short)l->rx.fd, (unsigned short)conn->handle.fd,
+			             pn, get_host_port(src), alpn);
 			break;
 		case AF_UNIX:
 			/* UNIX socket, only the destination is known */
 			chunk_printf(&trash, "%08x:%s.accept(%04x)=%04x from [unix:%d] ALPN=%s\n",
-			             s->uniq_id, fe->id, (unsigned short)l->fd, (unsigned short)conn->handle.fd,
+			             s->uniq_id, fe->id, (unsigned short)l->rx.fd, (unsigned short)conn->handle.fd,
 			             l->luid, alpn);
 			break;
 		}
@@ -136,28 +141,17 @@ int frontend_accept(struct stream *s)
 		s->req.flags |= CF_READ_DONTWAIT; /* one read is usually enough */
 
 	if (unlikely(fe->nb_req_cap > 0)) {
-		if ((s->req_cap = pool_alloc(fe->req_cap_pool)) == NULL)
+		if ((s->req_cap = pool_zalloc(fe->req_cap_pool)) == NULL)
 			goto out_return;	/* no memory */
-		memset(s->req_cap, 0, fe->nb_req_cap * sizeof(void *));
 	}
 
 	if (unlikely(fe->nb_rsp_cap > 0)) {
-		if ((s->res_cap = pool_alloc(fe->rsp_cap_pool)) == NULL)
+		if ((s->res_cap = pool_zalloc(fe->rsp_cap_pool)) == NULL)
 			goto out_free_reqcap;	/* no memory */
-		memset(s->res_cap, 0, fe->nb_rsp_cap * sizeof(void *));
 	}
 
-	if (fe->http_needed) {
-		/* we have to allocate header indexes only if we know
-		 * that we may make use of them. This of course includes
-		 * (mode == PR_MODE_HTTP).
-		 */
-		if (unlikely(!http_alloc_txn(s)))
-			goto out_free_rspcap; /* no memory */
-
-		/* and now initialize the HTTP transaction state */
-		http_init_txn(s);
-	}
+	if ((fe->http_needed || IS_HTX_STRM(s)) && !http_create_txn(s))
+		goto out_free_rspcap;
 
 	/* everything's OK, let's go on */
 	return 1;
@@ -222,9 +216,16 @@ smp_fetch_fe_defbe(const struct arg *args, struct sample *smp, const char *kw, v
 static int
 smp_fetch_fe_req_rate(const struct arg *args, struct sample *smp, const char *kw, void *private)
 {
+	struct proxy *px = args->data.prx;
+
+	if (px == NULL)
+		return 0;
+	if (px->cap & PR_CAP_DEF)
+		px = smp->px;
+
 	smp->flags = SMP_F_VOL_TEST;
 	smp->data.type = SMP_T_SINT;
-	smp->data.u.sint = read_freq_ctr(&args->data.prx->fe_req_per_sec);
+	smp->data.u.sint = read_freq_ctr(&px->fe_req_per_sec);
 	return 1;
 }
 
@@ -235,9 +236,16 @@ smp_fetch_fe_req_rate(const struct arg *args, struct sample *smp, const char *kw
 static int
 smp_fetch_fe_sess_rate(const struct arg *args, struct sample *smp, const char *kw, void *private)
 {
+	struct proxy *px = args->data.prx;
+
+	if (px == NULL)
+		return 0;
+	if (px->cap & PR_CAP_DEF)
+		px = smp->px;
+
 	smp->flags = SMP_F_VOL_TEST;
 	smp->data.type = SMP_T_SINT;
-	smp->data.u.sint = read_freq_ctr(&args->data.prx->fe_sess_per_sec);
+	smp->data.u.sint = read_freq_ctr(&px->fe_sess_per_sec);
 	return 1;
 }
 
@@ -248,9 +256,25 @@ smp_fetch_fe_sess_rate(const struct arg *args, struct sample *smp, const char *k
 static int
 smp_fetch_fe_conn(const struct arg *args, struct sample *smp, const char *kw, void *private)
 {
+	struct proxy *px = args->data.prx;
+
+	if (px == NULL)
+		return 0;
+	if (px->cap & PR_CAP_DEF)
+		px = smp->px;
+
 	smp->flags = SMP_F_VOL_TEST;
 	smp->data.type = SMP_T_SINT;
-	smp->data.u.sint = args->data.prx->feconn;
+	smp->data.u.sint = px->feconn;
+	return 1;
+}
+
+static int
+smp_fetch_fe_client_timeout(const struct arg *args, struct sample *smp, const char *km, void *private)
+{
+	smp->flags = SMP_F_VOL_TXN;
+	smp->data.type = SMP_T_SINT;
+	smp->data.u.sint = TICKS_TO_MS(smp->sess->fe->timeout.client);
 	return 1;
 }
 
@@ -259,12 +283,13 @@ smp_fetch_fe_conn(const struct arg *args, struct sample *smp, const char *kw, vo
  * Please take care of keeping this list alphabetically sorted.
  */
 static struct sample_fetch_kw_list smp_kws = {ILH, {
-	{ "fe_conn",      smp_fetch_fe_conn,      ARG1(1,FE), NULL, SMP_T_SINT, SMP_USE_INTRN, },
-	{ "fe_defbe",     smp_fetch_fe_defbe,     0,          NULL, SMP_T_STR,  SMP_USE_FTEND, },
-	{ "fe_id",        smp_fetch_fe_id,        0,          NULL, SMP_T_SINT, SMP_USE_FTEND, },
-	{ "fe_name",      smp_fetch_fe_name,      0,          NULL, SMP_T_STR,  SMP_USE_FTEND, },
-	{ "fe_req_rate",  smp_fetch_fe_req_rate,  ARG1(1,FE), NULL, SMP_T_SINT, SMP_USE_INTRN, },
-	{ "fe_sess_rate", smp_fetch_fe_sess_rate, ARG1(1,FE), NULL, SMP_T_SINT, SMP_USE_INTRN, },
+	{ "fe_client_timeout", smp_fetch_fe_client_timeout, 0,          NULL, SMP_T_SINT, SMP_USE_FTEND, },
+	{ "fe_conn",           smp_fetch_fe_conn,           ARG1(1,FE), NULL, SMP_T_SINT, SMP_USE_INTRN, },
+	{ "fe_defbe",          smp_fetch_fe_defbe,          0,          NULL, SMP_T_STR,  SMP_USE_FTEND, },
+	{ "fe_id",             smp_fetch_fe_id,             0,          NULL, SMP_T_SINT, SMP_USE_FTEND, },
+	{ "fe_name",           smp_fetch_fe_name,           0,          NULL, SMP_T_STR,  SMP_USE_FTEND, },
+	{ "fe_req_rate",       smp_fetch_fe_req_rate,       ARG1(1,FE), NULL, SMP_T_SINT, SMP_USE_INTRN, },
+	{ "fe_sess_rate",      smp_fetch_fe_sess_rate,      ARG1(1,FE), NULL, SMP_T_SINT, SMP_USE_INTRN, },
 	{ /* END */ },
 }};
 
