@@ -25,6 +25,7 @@
 #include <haproxy/chunk.h>
 #include <haproxy/global.h>
 #include <haproxy/http.h>
+#include <haproxy/http_ana-t.h>
 #include <haproxy/http_rules.h>
 #include <haproxy/log.h>
 #include <haproxy/pool.h>
@@ -80,12 +81,11 @@ struct act_rule *parse_http_req_cond(const char **args, const char *file, int li
 	const struct action_kw *custom = NULL;
 	int cur_arg;
 
-	rule = calloc(1, sizeof(*rule));
+	rule = new_act_rule(ACT_F_HTTP_REQ, file, linenum);
 	if (!rule) {
 		ha_alert("parsing [%s:%d]: out of memory.\n", file, linenum);
 		goto out_err;
 	}
-	rule->from = ACT_F_HTTP_REQ;
 
 	if (((custom = action_http_req_custom(args[0])) != NULL)) {
 		char *errmsg = NULL;
@@ -159,12 +159,11 @@ struct act_rule *parse_http_res_cond(const char **args, const char *file, int li
 	const struct action_kw *custom = NULL;
 	int cur_arg;
 
-	rule = calloc(1, sizeof(*rule));
+	rule = new_act_rule(ACT_F_HTTP_RES, file, linenum);
 	if (!rule) {
 		ha_alert("parsing [%s:%d]: out of memory.\n", file, linenum);
 		goto out_err;
 	}
-	rule->from = ACT_F_HTTP_RES;
 
 	if (((custom = action_http_res_custom(args[0])) != NULL)) {
 		char *errmsg = NULL;
@@ -239,12 +238,11 @@ struct act_rule *parse_http_after_res_cond(const char **args, const char *file,
 	const struct action_kw *custom = NULL;
 	int cur_arg;
 
-	rule = calloc(1, sizeof(*rule));
+	rule = new_act_rule(ACT_F_HTTP_RES, file, linenum);
 	if (!rule) {
 		ha_alert("parsing [%s:%d]: out of memory.\n", file, linenum);
 		goto out_err;
 	}
-	rule->from = ACT_F_HTTP_RES;
 
 	if (((custom = action_http_after_res_custom(args[0])) != NULL)) {
 		char *errmsg = NULL;
@@ -301,26 +299,6 @@ struct act_rule *parse_http_after_res_cond(const char **args, const char *file,
 	return NULL;
 }
 
-/* completely free redirect rule */
-void http_free_redirect_rule(struct redirect_rule *rdr)
-{
-	struct logformat_node *lf, *lfb;
-
-	if (rdr->cond) {
-		prune_acl_cond(rdr->cond);
-		free(rdr->cond);
-	}
-	free(rdr->rdr_str);
-	free(rdr->cookie_str);
-	list_for_each_entry_safe(lf, lfb, &rdr->rdr_fmt, list) {
-		LIST_DELETE(&lf->list);
-		release_sample_expr(lf->expr);
-		free(lf->arg);
-		free(lf);
-	}
-	free(rdr);
-}
-
 /* Parses a redirect rule. Returns the redirect rule on success or NULL on error,
  * with <err> filled with the error message. If <use_fmt> is not null, builds a
  * dynamic log-format rule instead of a static string. Parameter <dir> indicates
@@ -329,7 +307,7 @@ void http_free_redirect_rule(struct redirect_rule *rdr)
 struct redirect_rule *http_parse_redirect_rule(const char *file, int linenum, struct proxy *curproxy,
                                                const char **args, char **errmsg, int use_fmt, int dir)
 {
-	struct redirect_rule *rule = NULL;
+	struct redirect_rule *rule;
 	int cur_arg;
 	int type = REDIRECT_TYPE_NONE;
 	int code = 302;
@@ -390,7 +368,7 @@ struct redirect_rule *http_parse_redirect_rule(const char *file, int linenum, st
 				memprintf(errmsg,
 				          "'%s': unsupported HTTP code '%s' (must be one of 301, 302, 303, 307 or 308)",
 				          args[cur_arg - 1], args[cur_arg]);
-				goto err;
+				return NULL;
 			}
 		}
 		else if (strcmp(args[cur_arg], "drop-query") == 0) {
@@ -399,45 +377,48 @@ struct redirect_rule *http_parse_redirect_rule(const char *file, int linenum, st
 		else if (strcmp(args[cur_arg], "append-slash") == 0) {
 			flags |= REDIRECT_FLAG_APPEND_SLASH;
 		}
+		else if (strcmp(args[cur_arg], "ignore-empty") == 0) {
+			flags |= REDIRECT_FLAG_IGNORE_EMPTY;
+		}
 		else if (strcmp(args[cur_arg], "if") == 0 ||
 			 strcmp(args[cur_arg], "unless") == 0) {
 			cond = build_acl_cond(file, linenum, &curproxy->acl, curproxy, (const char **)args + cur_arg, errmsg);
 			if (!cond) {
 				memprintf(errmsg, "error in condition: %s", *errmsg);
-				goto err;
+				return NULL;
 			}
 			break;
 		}
 		else {
 			memprintf(errmsg,
-			          "expects 'code', 'prefix', 'location', 'scheme', 'set-cookie', 'clear-cookie', 'drop-query' or 'append-slash' (was '%s')",
+			          "expects 'code', 'prefix', 'location', 'scheme', 'set-cookie', 'clear-cookie', 'drop-query', 'ignore-empty' or 'append-slash' (was '%s')",
 			          args[cur_arg]);
-			goto err;
+			return NULL;
 		}
 		cur_arg++;
 	}
 
 	if (type == REDIRECT_TYPE_NONE) {
 		memprintf(errmsg, "redirection type expected ('prefix', 'location', or 'scheme')");
-		goto err;
+		return NULL;
 	}
 
 	if (dir && type != REDIRECT_TYPE_LOCATION) {
 		memprintf(errmsg, "response only supports redirect type 'location'");
-		goto err;
+		return NULL;
 	}
 
 	rule = calloc(1, sizeof(*rule));
-	if (!rule)
-		goto out_of_memory;
+	if (!rule) {
+		memprintf(errmsg, "parsing [%s:%d]: out of memory.", file, linenum);
+		return NULL;
+	}
 	rule->cond = cond;
 	LIST_INIT(&rule->rdr_fmt);
 
 	if (!use_fmt) {
 		/* old-style static redirect rule */
 		rule->rdr_str = strdup(destination);
-		if (!rule->rdr_str)
-			goto out_of_memory;
 		rule->rdr_len = strlen(destination);
 	}
 	else {
@@ -455,7 +436,7 @@ struct redirect_rule *http_parse_redirect_rule(const char *file, int linenum, st
 			cap |= (dir ? SMP_VAL_BE_HRS_HDR : SMP_VAL_BE_HRQ_HDR);
 		if (!(type == REDIRECT_TYPE_PREFIX && destination[0] == '/' && destination[1] == '\0')) {
 			if (!parse_logformat_string(destination, curproxy, &rule->rdr_fmt, LOG_OPT_HTTP, cap, errmsg)) {
-				goto err;
+				return  NULL;
 			}
 			free(curproxy->conf.lfs_file);
 			curproxy->conf.lfs_file = strdup(curproxy->conf.args.file);
@@ -470,15 +451,11 @@ struct redirect_rule *http_parse_redirect_rule(const char *file, int linenum, st
 		rule->cookie_len = strlen(cookie);
 		if (cookie_set) {
 			rule->cookie_str = malloc(rule->cookie_len + 10);
-			if (!rule->cookie_str)
-				goto out_of_memory;
 			memcpy(rule->cookie_str, cookie, rule->cookie_len);
 			memcpy(rule->cookie_str + rule->cookie_len, "; path=/;", 10);
 			rule->cookie_len += 9;
 		} else {
 			rule->cookie_str = malloc(rule->cookie_len + 21);
-			if (!rule->cookie_str)
-				goto out_of_memory;
 			memcpy(rule->cookie_str, cookie, rule->cookie_len);
 			memcpy(rule->cookie_str + rule->cookie_len, "; path=/; Max-Age=0;", 21);
 			rule->cookie_len += 20;
@@ -492,18 +469,6 @@ struct redirect_rule *http_parse_redirect_rule(const char *file, int linenum, st
 
  missing_arg:
 	memprintf(errmsg, "missing argument for '%s'", args[cur_arg]);
-	goto err;
- out_of_memory:
-	memprintf(errmsg, "parsing [%s:%d]: out of memory.", file, linenum);
- err:
-	if (rule)
-		http_free_redirect_rule(rule);
-	else if (cond) {
-		/* rule not yet allocated, but cond already is */
-		prune_acl_cond(cond);
-		free(cond);
-	}
-
 	return NULL;
 }
 
